!      subroutine DisturbIt (iyear)
!      SUBROUTINE clus_grw (iyear)
!      SUBROUTINE push (i,j)
!      SUBROUTINE popoff (i)
!      Subroutine Nei_Eval
!      subroutine Disturb_setup
!      subroutine mfthfm (H, simlen, wrkvec, iseed)
!      subroutine mpfm1d (X, N, maxlevel, sigma, H, Tbegin, Tend, iseed)
!      SUBROUTINE indexx(n,arr,inxx)

       subroutine DisturbIt (iyear)
!
       use repsblock
       use mapblock
       USE dstrb_blk
       USE specblock
!
       implicit none
       integer iyear, i, j, k, ii, jj, iloc, jloc, nx_cnt, inx
       integer acnt, bcnt       ! acnt -- disturbed this year
                                ! bcnt -- already empty sites
       INTEGER isc, iv_isc
       real ran1, pp
       REAL s_cdf(0:100)         ! cdf of disturbance severity

       s_cdf = 0.0
       if (max_sevr .le. 100) then
          iv_isc = max_sevr
         else
          iv_isc = 100
       endif

!  summarize already disturbed and count-down disturbance map
       bcnt = 0

       do j = 2, jcol+1
          do i = 2, irow+1
             if (map_sp(i,j) .eq. 0) bcnt = bcnt + 1
             if (freq_type .ne. 'N') then
                if (dstb_map(i,j) .gt. 0) then
                   dstb_map(i,j) = dstb_map(i,j) - 1
                end if
             end if
          end do
       end do

       pb = 1.0 - (float (bcnt) / float (irow*jcol)) ! proportion of map currently occupied
       rdinc = 1.0 / float (irow*jcol)

!  if no disturbances this year, then summarize and return
       if (idefect(iyear) .eq. 0) then
          do i = 2, irow+1
             do j = 2, jcol+1
!  set cumulative frequency distribution
!                isc = INT(float(dstb_map(i,j)) / float(max_sevr * iv_isc) + 0.9)
                if (freq_type .ne. 'N') then
                    isc = dstb_map(i,j)
                   else
                    isc = 0
                endif
                if (isc .lt. 0 .or. isc .gt. iv_isc) then      ! error check
                   PRINT *, 'ERROR: ', iyear, i,j,isc
                   stop
                end if
                s_cdf(isc) = s_cdf(isc) + 1.0
             end do
          end do
          do i = 0, iv_isc
             s_cdf(i) = s_cdf(i) / float(irow*jcol)
          end do
          acnt = 0
                     ! irep = replication number
                     ! iyear = simulation year
                     ! max_sevr = integer index from 0 - 100
                     ! idefect(iyear) = number of sites to be disturbed 
                     !                  in year iyear
                     ! acnt = sites disturbed this year
                     ! bcnt = already empty sites
                     ! pb = proportion of map occupied
                     ! s_cdf(i) = frequency count of sites experiencing 
                     !    disturbance severity from 1 - iv_isc
                     ! iv_isc = max_sevr (temporary variable)
   !      write (18,101) irep, iyear, max_sevr, idefect(iyear), bcnt, &
   !                     acnt, pb, (s_cdf(i), i=0,iv_isc)
   !      write (18,101) irep, iyear, idefect(iyear), pb

          return
       endif

!
!  two variables govern disturbance:
!      d_type    -- describes disturbance type ('R' = random and 'C' = clustered)
!      defect(i) -- a precalculated vector of yearly disturbance extent
!  in all cases disturbance sets species to zero on identified grid sites

!  species disturbances           
       select case (d_type)
           case ('R','r')  ! random, independent disturbances
              do k = 1, idefect(iyear)
88               i = INT(ran1(iseed_dstb) * (irow-1) + 0.5) + 2
                 j = INT(ran1(iseed_dstb) * (jcol-1) + 0.5) + 2
                 if (map_sp(i,j) .le. 0) GOTO 88      ! only disturb occupied sites
                 map_sp(i,j) = -1
                 pb = pb - rdinc                      ! proportion of map occupied
                 if (pb .le. 0.025) goto 89
              end do
89            continue

           case ('C','c')  ! grow disturbances like an epidemic
              call clus_grw (iyear)

           case default
              write (15,*) 'Disturbances not simulated'
              return
       end select
!  summarize disturbances
       acnt = 0
       do i = 2, irow+1
          do j = 2, jcol+1
             if (map_sp(i,j) .lt. 0) then             ! when species disturbed
                acnt = acnt + 1                       ! count disturbances - empty sites
!  habitat disturbance based on disturbance severity
                nx_cnt = 0                            ! examine immediate neighborhood
                inx = 0                               ! neighbor counter
                do ii = i-2,i+2                       ! count number of disturbed neighbors
                   do jj = j-2,j+2                    !   index ii, jj set to 25 neighbors

                       iloc = ii
                       jloc = jj
                       select case (movtype)
                           case ('W') ! wrap edges
                             if (ii .lt. 2) iloc = irow + ii
                             if (ii .gt. irow+1) iloc = ii - irow
                             if (jj .lt. 2) jloc = jj + jcol
                             if (jj .gt. jcol+1) jloc = jj - jcol

                             if (map_sp(iloc,jloc) .LT. 0) nx_cnt = nx_cnt + 1
                             inx = inx + 1

                           case ('A','R') ! absorbing or reflecting boundaries
                             if (ii .ge. 2 .and. ii .le. irow+1 .or. &
                                 jj .ge. 2 .AND. jj .le. jcol+1) then
                                 if (map_sp(iloc,jloc) .LT. 0) nx_cnt = nx_cnt + 1
                                 inx = inx + 1
                             endif

                           case default
                             if (map_sp(ii,jj) .LT. 0) nx_cnt = nx_cnt + 1
                             inx = inx + 1
                       end select

                   end do
                end do
! estimate disturbance severity based on the proportion 
!  of disturbed neighbors (pp)
!    nx_cnt = count of disturbed neighbors
!    inx = total number of neighbors (25 potential neighbors)
                pp = float (nx_cnt) / float (inx)
!               PRINT *, 'i,j,pp,nx_cnt,inx', i, j , pp, nx_cnt, inx
!  increment disturbance map (disturbances are additive)
                dstb_map(i,j) = dstb_map(i,j) + int(pp * max_sevr) + 1
                if (dstb_map(i,j) .gt. max_sevr) dstb_map(i,j) = max_sevr
!               PRINT *, 'dstb_map: ', dstb_map(i,j)

             end if

!  set cumulative frequency distribution
!             isc = INT(float(dstb_map(i,j)) / float(max_sevr * iv_isc) + 0.9)
             isc = dstb_map(i,j)
             if (isc .lt. 0 .or. isc .gt. iv_isc) then
                PRINT *, 'ERROR: ', iyear, i, j, isc, dstb_map(i,j)
                stop
             end if
             s_cdf(isc) = s_cdf(isc) + 1.0
          end do
       end do
       do i = 0, iv_isc
          s_cdf(i) = s_cdf(i) / float(irow*jcol)
       end do

!      PRINT 100, iyear, idefect(iyear), bcnt, acnt, acnt+bcnt, pb
100    format ('Yr(',i3,') req: ',i8,' before',i8,' disturbed',i8, &
               ' total ',i8,' pb = ',f6.4)
!       write (18,101) irep, iyear, idefect(iyear), pb
!       write (18,101) irep, iyear, max_sevr, idefect(iyear), bcnt, acnt, pb, &
!                        (s_cdf(i), i=0,iv_isc)
101    format (3i6,1x,f8.5)
!101    format (i3, i4, 4(1x,i8),100(1x,f8.5))

!  reset map_sp from negative # to zero
       do i = 2, irow+1
          do j = 2, jcol+1
             if (map_sp(i,j) .lt. 0) map_sp(i,j) = 0
          end do
       end do

       return
       end
!
!  routine for growing disturbance as a spreading cluster
       SUBROUTINE clus_grw (iyear)
!
       use repsblock
       use mapblock
       USE dstrb_blk
!
       integer iyear, i, j, icnt
       real ran1
!
!  idnt -- counts the number of disturbed sites
       idnt = 0
!  pick a random start location for cluster growth
       icnt = 0                                   
99       i = INT(ran1(iseed_dstb) * (irow-1) + 0.5) + 2
         j = INT(ran1(iseed_dstb) * (jcol-1) + 0.5) + 2
         icnt = icnt + 1
! quit if no starting point found
         if (icnt .gt. irow*jcol) then
            WRITE(15,*) 'Could not locate starting point for disturbance'
            write (15,100) iyear, idefect(iyear), icnt, idnt, pb
            return
         end if
                                            ! loop if starting point not valid
       if (map_sp(i,j) .le. 0) GOTO 99      ! only disturb occupied sites
       nstack = 0
       call push (i,j)

!  check to see if done
       if (idnt .eq. idefect(iyear) .or. pb .le. 0.025) then
!        write (15,100) iyear, idefect(iyear), icnt, idnt, pb
         return
       endif

!  start cluster growth
       do WHILE (nstack .GT. 0)
!  spin probability to disturb
          call nei_eval
!
          if (idnt .ge. idefect(iyear) .or. pb .le. 0.025) then
!                 write (15,100) iyear, idefect(iyear), icnt, idnt, pb
100             format (' Yr ',i5,' Dstrb reqstd ',i8,' starts ',i6,' simltd ',i8,' pb = ',g12.6)
                return
          endif
!  pop inspected site off list
!          call popoff (1)
           call popoff 
89     end do   ! end of while loop

!  loop back to start another epidemic
       GOTO 99
!
       END subroutine

       SUBROUTINE push (i,j)
       use repsblock
       use mapblock
       USE dstrb_blk
       INTEGER i, j

       map_sp(i,j) = -1
       nstack = nstack + 1
       istack(nstack) = i
       jstack(nstack) = j
       idnt = idnt + 1
       pb = pb - rdinc
!
       return
       end

       SUBROUTINE popoff 
       USE dstrb_blk
       INTEGER i
!
       do i = 2,nstack
          istack(i-1) = istack(i)
          jstack(i-1) = jstack(i)
       end do
       nstack = nstack - 1
!
       END SUBROUTINE

       Subroutine Nei_Eval 
!
       USE parmlist
       USE mapblock
       USE dstrb_blk
!
       integer i, j, inc, jnc
!  consider the 8 neighbors around current (i,j) site
!     also wrapping boundaries
       do inc = -1,1
          i = istack(1) + inc
          if (i .gt. irow+1) i = 2
          if (i .lt. 2) i = irow + 1
          do jnc = -1,1
             j = jstack(1) + jnc
             if (j .gt. jcol+1) j = 2
             if (j .lt. 2) j = jcol + 1
!  if site has not been disturbed, add to stack
             if (map_sp(i,j) .gt. 0) then
                if (ran1(iseed_dstb) .le. px) then
                   call push (i,j)
                END if
             end if
!         write (15,100) nstack, i, j, map_sp(i,j)
100       format ('nei-nstack,i,j,map',4i5)
          end do
       end do
!
       return
       end
!
       subroutine Disturb_setup
       use parmlist
       USE mapblock
       USE repsblock
       USE dstrb_blk
!
       implicit none
       integer inxx(maxsim)
       INTEGER ttl_yrs, ttl_extent, idx, isum_xcss, isum_dist
       integer i_up_lim, iadj_sum, iadj, isumd, isumin, isumax, iavg
       INTEGER irs, ncycle, idinc, i, icnt, jcnt, ifreq, ii
       REAL ttl, factor, ran1, Gauss, pct
       real wrkvec(maxsim), gx(maxsim), gy(maxsim)
!
! this routine sets up the disturbance regime
!   variables set on input
!     fract_per_yr -- fraction of map disturbed per year (average)
!        range = 0.0 --> 0.99
!     fract_years  -- fraction of years to be disturbed
!     idur -- duration (in years) of each disturbance event
!             used for square and sine
!
!   calculate total extent to be disturbed

!  temporary setting of variables

!   pick a start year
       irs = INT(simlen * p_irs + 0.999)        ! p_irs constant - 0.05
       if (irs .gt. 5) irs = 5
!   calculate total number of years to be disturbed
       ttl_yrs = INT(fract_years * (simlen - 2*irs) + 0.99)

!   calculate total extent of disturbances
       ttl_extent = int (fract_per_yr * (simlen - 2*irs) * (irow*jcol) + 0.5)

!   initialize idefect
       idefect = 0
!   zero out disturbance map
       if (freq_type .ne. 'N') dstb_map = 0

!   set up disturbance vector based on frequency of disturbance
       select case (freq_type)
!
          case ('F','f')  !  square wave
!   ncycles -- number of cycles
            ncycle = (ttl_yrs) / idur
!   ifreq -- length of disturbance cycle
            ifreq = (simlen) / ncycle
!   set each disturbance event to the average
            idinc = ttl_extent / ttl_yrs
!   begin disturbance cycles
            jcnt = 0
            do i = irs+1, simlen-irs
              if (MOD(i-irs,ifreq) .LE. idur-1 .and. jcnt .lt. ttl_yrs) then
                 idefect(i) = idinc
                 jcnt = jcnt + 1
               else
                 idefect(i) = 0
              end if
            end do
!
          case ('S','s')  !  sine-wave
!   ncycles -- number of cycles
!   ifreq -- length of disturbance cycle
            ncycle = (ttl_yrs) / idur
            ifreq = (simlen) / ncycle
!   begin disturbance cycles
            ttl = 0.0
            factor = 3.14159 / float(idur)
            jcnt = 0
            do i = irs+1, simlen
              ii = MOD(i-irs,ifreq)
              if (ii .LE. idur .and. jcnt .lt. ttl_yrs) then
                 if (ii .gt. 0) then
                    wrkvec(i) = SIN((float(ii)-0.5) * factor)
                    jcnt = jcnt + 1
                 end if
               else
                 wrkvec(i) = 0.0
              end if
              ttl = ttl + wrkvec(i)
            end do
!
!   no disturbance for the first 'irs' years
            factor = float(ttl_extent) / ttl
            do i = irs+1, simlen
               idefect(i) = INT(wrkvec(i) * factor + 0.5)
            end do
!
          case ('R','r')  !  random but regular
!   sample the years to disturb without replacement
            do i = 1,simlen-irs
               wrkvec(i) = ran1(iseed_dstb)
            end do
!   sort by index
            call indexx (simlen-irs, wrkvec, inxx)
!   find number largest values to = ttl_yrs
            icnt = 0
            ttl = 0.0
            do i = simlen-irs, 1, -1
               icnt = icnt + 1
               if (icnt .gt. ttl_yrs) wrkvec(inxx(i)) = 0.0
               ttl = ttl + wrkvec(inxx(i))
            end do
!   move to idefect(i) by setting equal to idinc and remainder to 0.0
            factor = float(ttl_extent) / float(ttl_yrs)

            do i = 1, simlen-irs
               if (wrkvec(i) .gt. 0.0) idefect(i+irs) = factor
            end do
!
          case ('W','w')  !  white noise -- random, irregular
!   sample the years to disturb without replacement
            do i = 1,simlen-irs
               wrkvec(i) = ran1(iseed_dstb)
            end do
!   sort by index
            call indexx (simlen-irs,wrkvec,inxx)
!   find number largest values to = ttl_yrs
            icnt = 0
            do i = simlen-irs, 1, -1
               icnt = icnt + 1
               if (icnt .gt. ttl_yrs) wrkvec(inxx(i)) = 0.0
            end do
!   renormalize here so that mean and variance of disturbance
!     can be controlled
            ttl = 0.0
            DO i = 1, simlen-irs
               IF (wrkvec(i) .GT. 0.0) then
!                 wrkvec(i) = Gauss (1.0, dcv, iseed_dstb)
2                 wrkvec(i) = Gauss (1.0, dcv, iseed_dstb)
                  if (wrkvec(i) .le. 0.0) goto 2
               endif
               ttl = ttl + wrkvec(i)
            enddo
!   estimate adjustment factor
            factor = float(ttl_extent) / ttl
!   adjust disturbance fector
            idx = 0
            do i = 1, simlen-irs
               idefect(i+irs) = INT(wrkvec(i) * factor + 0.5)
               idx = idx + idefect(i+irs)
            end do
!
          case ('B','b')  ! fractial Brownian motion - 1d algorithm
                          ! from midpoint displacement method of Pietgen and
                          !   Saupe 1988
            call mfthfm (dH, simlen-irs, wrkvec, iseed_dstb)
!   sort by index
            call indexx (simlen-irs, wrkvec, inxx)
!   find number largest values to = ttl_yrs
            icnt = 0
            do i = simlen-irs, 1, -1
               icnt = icnt + 1
        !      PRINT *, i, inxx(i), wrkvec(inxx(i))
               if (icnt .gt. ttl_yrs) then
                  wrkvec(inxx(i)) = 0.0
                else
                  wrkvec(inxx(i)) = 1.0
               endif
        !      PRINT *, i, inxx(i), wrkvec(inxx(i))
            end do
!   renormalize here so that mean and variance of disturbance
!     can be controlled
            ttl = 0.0
            DO i = 1, simlen-irs
               IF (wrkvec(i) .GT. 0.0) then
3                 wrkvec(i) = Gauss (1.0, dcv, iseed_dstb)
                  if (wrkvec(i) .le. 0.0) goto 3
               endif
               ttl = ttl + wrkvec(i)
            enddo
!   move to idefect(i) by normalizing numbers
            factor = float(ttl_extent) / ttl
!   pick amounts of disturbance by normalizing vector
            do i = 1, simlen-irs
               idefect(i+irs) = INT(wrkvec(i) * factor + 0.5)
          !    print *, ' idefect ',i+irs, idefect(i+irs)
            end do

          case ('N','n')
             PRINT *, 'No disturbances simulated.'
             do i = irs+1, simlen
                idefect(i) = 0
             end do
          case default
             PRINT *, 'Disturbance type not recognized.'
             PRINT *, 'No disturbances simulated.'
             do i = irs+1, simlen
                idefect(i) = 0
             end do
       end select      ! freq_type

!  adjust disturbances to fit within constraints
       isum_xcss = 0.0
       isum_dist = 0.0
       i_up_lim  = irow * jcol
       do i = irs+1, simlen
          if (idefect(i) .gt. i_up_lim) then
             isum_xcss  = idefect(i) - i_up_lim + isum_xcss
             idefect(i) = i_up_lim
           end if
          if (idefect(i) .gt. 0) isum_dist  = i_up_lim - idefect(i) + isum_dist
       end do

!  adjust idefect if necessary

       iadj_sum = 0
       if (isum_xcss .gt. 0) then
          do i = irs+1, simlen
             if (idefect(i) .gt. 0) then
               iadj = INT((float(i_up_lim - idefect(i)) / &
                      float(isum_dist) * float(isum_xcss)) + 0.5)
               idefect(i) = iadj + idefect(i)
               iadj_sum = iadj_sum + iadj
             end if
          end do
       end if
!
!  summary statistics
       isumd=0
       isumin=2*irow*jcol
       isumax=0
       do i = 1,simlen
          isumd = isumd + idefect(i)
          if (isumin .gt. idefect(i)) isumin = idefect(i)
          if (isumax .lt. idefect(i)) isumax = idefect(i)
       end do
       iavg = isumd / (simlen-irs)
       pct = float(isumd) / float(irow*jcol*(simlen-irs))
       PRINT 200, iavg, isumd, pct, isumin, isumax
200    format (/t10,'Disturbance summary:'/t12,'Average: ',t25,i10/ &
               t12,'Total extent:',t25,i10, t40,'fract/yr ',f6.4/   &
               t12,'Minimum:',t25,i10/ t12,'Maximum:'t25,i10)
!
!  prepare for visualization
       do i = 1, simlen
          gx(i) = i
          gy(i) = idefect(i)
       end do
!      IF (look) call line_graph (gx,gy,simlen)
!
       return
       end
!
       subroutine mfthfm (H, simlen, wrkvec, iseed)
!
       use parmlist
       INTEGER, parameter :: maxN=2**20+1
!
       integer iseed, simlen, N, maxlevel, i
       real X(0:maxN), wrkvec(maxsim), H, sigma
       REAL Tbegin, Tend
!       real Gauss
!
       N = 2
       maxlevel = 1
       do while (N .lt. simlen)
          N = 2**maxlevel
          maxlevel = maxlevel + 1
       enddo

       if (N .gt. maxN) then
           print 99, N, maxN
99         format ('  LENGTH OF TRANSECT IS ',I8,' WHICH EXCEEDS',  &
     'MAXIMUM ALLOWABLE LENGTH OF ',I8)
           goto 1
       endif
!
       sigma = 1.0
       Tbegin = 0.0
!       Tbegin = Gauss (0.0, 1.0, iseed)
       Tend = 0.0
!       Tend = Gauss (0.0, 1.0, iseed)
!
!  call 1d multi-fractal generator
       call mpfm1d (X, N, maxlevel, sigma, H, Tbegin, Tend, iseed)
       do i = 1,N+1
          wrkvec(i) = X(i-1)
       end do
!
       return
!
1      DO i = 1,simlen
         wrkvec(i) = 0.0
       enddo

       return
       end
!
       subroutine mpfm1d (X, N, maxlevel, sigma, H, Tbegin, Tend, iseed)
!
       implicit none
       INTEGER, parameter :: maxN=2**20+1
       real X(0:maxN), sigma, delta, Tbegin, Tend, H, f2, x0, x1, Gauss
       integer N, maxlevel, DD, d, iseed, is, ix
!
!  statment function definition
       f2(delta,x0,x1) = (x0+x1) / 2.0 + delta * Gauss(0.0,sigma,iseed)
!
       delta = sigma
       X(0) = Tbegin
       X(N) = Tend
!
       DD = N
       d = N / 2
!
       do is = 1, maxlevel
!  going from grid type I to type II
         delta = delta * 0.5**(0.5 * H)
         do ix = d, N-d, DD
            X(ix) = f2(delta, X(ix+d), X(ix-d))
         enddo
!  going from grid type II to type I
         delta = delta * 0.5**(0.5 * H)
         do ix = DD, N-d, DD
            X(ix) = f2(delta, X(ix+d), X(ix-d))
         enddo
         DD = DD / 2
         d = d / 2
       enddo
!
       return
       end
!
!  index sort routine from Numerical Recipes
      SUBROUTINE indexx(n,arr,inxx)
!  Indexes an array arr(1:n), i.e., outputs the array inxx(1:n) such that
!   arr(inxx(j) is in ascending order for j = 1,2,...,N.  The input quantities
!   n and arr are not changed
      INTEGER n,inxx(n),M,NSTACK
      REAL arr(n)
      PARAMETER (M=7,NSTACK=50)
      INTEGER i,inxxt,ir,itemp,j,jstack,k,l,istack(NSTACK)
      REAL a
      do 11 j=1,n
        inxx(j)=j
11    continue
      jstack=0
      l=1
      ir=n
1     if(ir-l.lt.M)then
        do 13 j=l+1,ir
          inxxt=inxx(j)
          a=arr(inxxt)
          do 12 i=j-1,1,-1
            if(arr(inxx(i)).le.a)goto 2
            inxx(i+1)=inxx(i)
12        continue
          i=0
2         inxx(i+1)=inxxt
13      continue
        if(jstack.eq.0)return
        ir=istack(jstack)
        l=istack(jstack-1)
        jstack=jstack-2
      else
        k=(l+ir)/2
        itemp=inxx(k)
        inxx(k)=inxx(l+1)
        inxx(l+1)=itemp
        if(arr(inxx(l+1)).gt.arr(inxx(ir)))then
          itemp=inxx(l+1)
          inxx(l+1)=inxx(ir)
          inxx(ir)=itemp
        endif
        if(arr(inxx(l)).gt.arr(inxx(ir)))then
          itemp=inxx(l)
          inxx(l)=inxx(ir)
          inxx(ir)=itemp
        endif
        if(arr(inxx(l+1)).gt.arr(inxx(l)))then
          itemp=inxx(l+1)
          inxx(l+1)=inxx(l)
          inxx(l)=itemp
        endif
        i=l+1
        j=ir
        inxxt=inxx(l)
        a=arr(inxxt)
3       continue
          i=i+1
        if(arr(inxx(i)).lt.a)goto 3
4       continue
          j=j-1
        if(arr(inxx(j)).gt.a)goto 4
        if(j.lt.i)goto 5
        itemp=inxx(i)
        inxx(i)=inxx(j)
        inxx(j)=itemp
        goto 3
5       inxx(l)=inxx(j)
        inxx(j)=inxxt
        jstack=jstack+2
        if(jstack.gt.NSTACK)pause 'NSTACK too small in indexx'
        if(ir-i+1.ge.j-l)then
          istack(jstack)=ir
          istack(jstack-1)=i
          ir=j-1
        else
          istack(jstack)=j-1
          istack(jstack-1)=l
          l=i
        endif
      endif
      goto 1
      END

