!..genmap.f90 contains the following routines for generating random maps
!   revision 11/29/04
!        Subroutine GenMap
!        Subroutine InputMap
!        Subroutine OutputMap ()
!        Subroutine RandMapX
!        Subroutine RandMapY
!        Subroutine InitSpc
!..
       Subroutine GenMap
       USE mapblock
       USE repsblock
!
       INTEGER i, j, ii, level
       REAL xs, prange
!
!..set entire map to zero
       map_hab = 0
!
       select case (maptype)
           case ('R')      !..simple random map -- with replacement
             Call RandMapY
           case ('S')      !..random map -- without replacement
             Call RandMapX
           case ('M')      !..multifractal map
             Call hfract
           case ('G')      !..multifractal map with a gradient
             Call hfract
           case ('I')      !..input map
             Call InputMap
!
       end select
!..generate map classes for random maps based on initial probabilities
       if (maptype .eq. 'R' .or. maptype .eq. 'S') then
          prange = p(ihab) - p(0)
          do i = 2, irow+1
             do j = 2, jcol+1
                if (map_hab(i,j) .gt. 0) then
!                  nxs = int(ran1(iseed_map)*ihab) + 1
                   xs = ran1(iseed_map) * prange + p(0)
                   do ii = 1, ihab
                      if (xs .gt. p(ii-1)) map_hab(i,j) = ii
                   enddo 
                endif
             enddo
          enddo
       endif
!
!..set borders of all maps to -9
3      do i = 1, irow+2
          map_hab(i,1) = -9
          map_hab(i,jcol+2) = -9
       enddo
!
       do j = 1, jcol+2
          map_hab(1,j) = -9
          map_hab(irow+2,j) = -9
       enddo
!
       return
       end
!
!..
       Subroutine OutputMap (iunit)
       USE mapblock
!
       integer iunit, imax, i, j
!  evaluate largest unit in sp_map
       imax = 0
       do i = 2, irow+1
          do j = 2, jcol+1
             IF (map_sp(i,j) .GT. imax) imax = map_sp(i,j)
          end do
       end do
!..output map
       select case (imax)
          case (1:9)
             do i = 2,irow+1
               write(iunit,100) (map_sp(i,j),j=2,jcol+1)
             enddo
          case (10:99)
             do i = 2,irow+1
               write(iunit,101) (map_sp(i,j),j=2,jcol+1)
             enddo
          case (100:999)
             do i = 2,irow+1
               write(iunit,102) (map_sp(i,j),j=2,jcol+1)
             enddo
          case (1000:)
             do i = 2,irow+1
               write(iunit,104) (map_sp(i,j),j=2,jcol+1)
             enddo
       end select
       write (iunit,99)
!
       return
99     format (' ')
100    format (500(i2))
101    format (500(i3))
102    format (500(i4))
104    format (500(i6))
       end
! **
       Subroutine GradMap
!
       USE mapblock
       USE repsblock
!
       pmin = 0.0
       pmax = 1.0
       pinc = (pmax - pmin) / (jcol-1)
       pt = pmin
!
       do j = 2, jcol+1
          do i = 2, irow+1
             if (ran1(iseed_map) .le. pt) map_hab(i,j) = 1
          enddo
          pt = pt + pinc
       enddo
!
       return
       end
!..
       Subroutine RandMapX
!  generate simple random maps
       USE mapblock
       USE repsblock
!
          do i = 2, irow+1
             do j = 2, jcol+1
                y=ran1(iseed_map)
                if (y .le. pr) map_hab(i,j) = 1
             enddo
          enddo
       return
       end
!
!..
       Subroutine RandMapY
!..this routine generates a random map by sampling without replacement
       USE parmlist
       USE mapblock
       USE repsblock
!
       integer non, iset
!
! calculate 'non' -- the number of sites to be "operated" on
       iset = 1
       non = int (pr * irow * jcol + 0.5)
       m_vec = irow * jcol
       if (non .gt. m_vec/2) then
          non = m_vec - non
          iset = 0
          do i = 2, irow+1
             do j = 2, jcol+1
                map_hab(i,j) = 1
             end do
          end do
       end if
       do i = 1, m_vec
          indx(i) = i
       enddo
!
!..generate random numbers
       do k = 1 , non
         ij  = int (ran1(iseed_map) * m_vec) + 1
!..locate within vector
         i = (indx(ij) - 1) / jcol + 1
         j = mod (indx(ij), jcol)
         if (j .eq. 0) j = jcol
         map_hab (i+1,j+1) = iset
!..shift vector down
         do ll = ij, m_vec-1
            indx(ll) = indx(ll+1)
         enddo
         m_vec = m_vec - 1
       enddo
!
       return
       end
!
!..
      Subroutine InputMap
!..assumes map is space delimited, ASCII file
      USE parmlist
      USE mapblock
      USE repsblock
!
!..input map from a data file
      if (irep .eq. 1) open (14, file=mapfile,status='old')
      do i = 2,irow+1
        read (14,*) (map_hab(i,j),j=2,jcol+1)
      enddo
!
      if (irep .eq. nrep) close(14)
!
      return
      end
!
       Subroutine InitSpc
!
       USE repsblock
       USE mapblock
       USE specblock

       implicit none
       integer :: i, j, nsite, iloc, jloc, ichoice, jopt, k, jhalf
       INTEGER :: iplace(0:maxsp), left, right
       real :: ran1, xchoice
!
!  set species map borders to zero
       map_sp = 0
!
       select case (mapinit)
          
       case ('A','a') !   place randomly based on off-map abundance,
                      !       ab_cdf, and distance, distance

!  initialize off-site aboundance vector as an inverse power function
         ab_cdf = 0
         do i = 1,ispecies
            if (alpha .eq. 0.0 .or. alpha .eq. 1.0) then
               ab_cdf(i) = (1.0 / float(i)**alpha) + ab_cdf(i-1)
              else
              ab_cdf(i) = (2.0-alpha)*alpha**i/ float(i) + ab_cdf(i-1)
           endif
         enddo
!    normalize cumulative frequency distribution
         do i = 1,ispecies
            ab_cdf(i) = ab_cdf(i) / ab_cdf(ispecies) 
         enddo
         
         print 100, ispecies
         write (15,100) ispecies
100      format (/t10,'Species off-site abundances for',i3,' species'/&
                  t10,'Species  Probability  Cum. Probability')
         print 101, (i,ab_cdf(i)-ab_cdf(i-1),ab_cdf(i),i=1,ispecies)
         write (15,101) (i,ab_cdf(i)-ab_cdf(i-1),ab_cdf(i),i=1,ispecies)
101      format (t12,'(',i2,')',3x,g12.6,2x,g12.6)
         print 102, (ab_cdf(ispecies)-ab_cdf(ispecies-1))*irow*jcol
         write (15,102) (ab_cdf(ispecies)-ab_cdf(ispecies-1))*irow*jcol
102      format (t12,'On map least probable # = ',g12.6)
         print 103, &
            (ab_cdf(ispecies)-ab_cdf(ispecies-1))*irow*jcol*100
         write (15,103) &
            (ab_cdf(ispecies)-ab_cdf(ispecies-1))*irow*jcol*100
103      format (t12,'Least probable # on Jm  = ',g12.6)


!  randomly populate 5% of sites based on off-site abundances      
         nsite = 0.05 * irow*jcol
         do i = 1, nsite
             iloc = int(ran1(iseed_mov) * irow) + 2
             jloc = int(ran1(iseed_mov) * jcol) + 2
             xchoice = ran1(iseed_mov)
             ichoice = 1
             do j = 2, ispecies
                if (xchoice .lt. ab_cdf(j) .and. xchoice .ge. ab_cdf(j-1)) then
                    ichoice = j
                endif
             enddo
             map_sp(iloc,jloc) = ichoice
        enddo

       case ('O','o') !   place species on all optimal sites 
                     !   decide between species for optimum locations
          iplace(0) = 0
          do j = 1, ihab
             if (iopt(1,j) .gt. 0) then
                iplace(j) = 1
                jopt = iopt(1,j)
             else
                iplace(j) = 0
                jopt = 0
             end if

          do k = 2, ispecies
             if (iopt(k,j) .gt. jopt) then
                iplace(j) = k
                jopt = iopt(k,j)
             endif
             if (iopt(k,j) .eq. jopt) then
                if (ran1(iseed_mov) .ge. 0.5) then
                    iplace(j) = k
                    jopt = iopt(k,j)
                endif
             end if
          end do  ! k loop -- ispecies
          end do  ! j loop -- ihab

!  place species on the map
          do j = 2, jcol+1
             do i = 2, irow+1
                map_sp(i,j) = iplace(map_hab(i,j))
             enddo
          enddo
!
       case ('L','l')
!   place species randomly on map edge without regard to habitat type
          do i = 2, irow+1
             map_sp(i,2) = int(ran1(iseed_mov) * ispecies + 1.0)
             map_sp(i,jcol+1) = int(ran1(iseed_mov) * ispecies + 1.0)
          enddo
!
       case ('R','r')
!   place species randomly on map without regard to habitat type
          do j = 2, jcol+1
             do i = 2, irow+1
                map_sp(i,j) = int(ran1(iseed_mov) * ispecies + 1.0)
             enddo
          enddo
!
       case ('E','e')
!   place 2 species on ends of map
          left = ispecies / 2
          right = ispecies - left
          do i = 2, irow+1
             map_sp(i,2) = int(ran1(iseed_mov) * left + 1.0)
             map_sp(i,jcol+1) = int(ran1(iseed_mov) * right + left + 1.0)
          enddo
!
       case ('H','h')
!   place 2 species on 1/2 of map
          jhalf = (jcol+2) / 2
          do i = 2, irow+1
             do j = 2, jhalf
                map_sp(i,j) = 1
                map_sp(i,j+jhalf-1) = 2
             enddo
          enddo
       end select
!
       return
       end

