!..hfract.f90 contains the following routines for generating fractal maps
!        function Gauss ()
!        subroutine hfract ()
!        subroutine mpfm2d ()
!        subroutine sort ()
!..
       subroutine hfract 
!
!   program written from algorithm MidPointFM2D in the 
!     Science of Fractal Images, M.F. Barnsley, R. L. Devaney
!     B. B. Mandelbrot, H.-O. Peitgen, D. Saupe, and R. F. Voss
!     Springer-Verlag
!         
       USE mapblock
       USE repsblock
       use fractblock
!
       integer icnt, i, j, ii, jj
       real    xmin, xmax, xmean, xsd
       logical addition
!
       N = 2**maxlevel
       if (N .gt. maxprm) stop
!
       addition = .true.
!
!  call 2d multi-fractal generator
       call mpfm2d (addition, iseed)
!
!  initialize statistics
       xmin = 1e30
       xmax = -1e30
       xmean = 0.0
       xsd = 0.0
       icnt = 0
!
!  obtain statistics 
       do i = 0,N
         do j = 0,N
            if (Xh(i,j) .lt. xmin) xmin = Xh(i,j)
            if (Xh(i,j) .gt. xmax) xmax = Xh(i,j)
            xmean = xmean + Xh(i,j)
            xsd = xsd + Xh(i,j)*Xh(i,j)
            icnt = icnt+1
         enddo
       enddo
       xsd = sqrt((xsd - (xmean*xmean)/icnt) / (icnt-1))
       xmean = xmean / icnt
!       print 200, xmean, xsd, xmin, xmax
200    format (/10x,'Hfract Summary'/10x,'Mean =    ',g12.6  &
        /10x,'St.dev. = ',g12.6 /10x,'Minimum = ',g12.6      &
        /10x,'Maximum = ',g12.6)
!
!  place Xh into cfd and sort
        icnt = 0
        do i = 0,N
           do j = 0,N
              icnt = icnt + 1
              cfd(icnt) = Xh(i,j)
            enddo
         enddo
         call sort (icnt, cfd)
!
!  obtain criterion values from cfd
        xcrit(ihab) = cfd(icnt)
        do i = 0, ihab-1
           icrit = int(p(i) * icnt) + 1
           xcrit(i) = cfd(icrit)
        enddo
! 
!  move Xh vector into map_hab simultaneously converting real numbers
!   in Xh to approprate integers in map_hab
       do i = 0, ihab
          kknt(i) = 0
       enddo
       do i = 0,N
          do j = 0,N
             ii = i + 2
             jj = j + 2
             map_hab(ii,jj) = 0
             do ij = 1,ihab
                if (Xh(i,j) .ge. xcrit(ij-1) .and. Xh(i,j) .le. xcrit(ij))  &
                     map_hab(ii,jj) = ij
             enddo
             kknt(map_hab(ii,jj)) = kknt(map_hab(ii,jj)) + 1
          enddo
       enddo
!
!  output map summary
!       print 299
299    format (/t10,'Hfract Map Summary:'/t12,'Habitat',5x,    &
          'P',10x,'Realized')
       pfract = 0.0
       do i = 0,ihab
          pfract = float(kknt(i)) / float(icnt) + pfract
!          print 300, i, p(i), pfract
300       format (t10,i6, 4x, g12.6, 2x, g12.6)
       enddo
!
       return
       end
!
       subroutine mpfm2d (addition, iseed)
!              
       USE parmlist
       use fractblock
       USE mapblock
!
       INTEGER  DD, d
       logical  addition

!  statment function definition
       f3(delta,x0,x1,x2) = (x0+x1+x2) / 3 + delta * Gauss(0.0,sigma,iseed)
       f4(delta,x0,x1,x2,x3) = (x0+x1+x2+x3) / 4 + delta * Gauss(0.0,sigma,iseed)
!
       N = 2**maxlevel
       delta = sigma
       Xh(0,0) = 0.0
       Xh(0,N) = 0.0
       Xh(N,0) = 0.0
       Xh(N,N) = 0.0
!      Xh(0,0) = delta * Gauss(0.0,sigma)
!      Xh(0,N) = delta * Gauss(0.0,sigma)
!      Xh(N,0) = delta * Gauss(0.0,sigma)
!      Xh(N,N) = delta * Gauss(0.0,sigma)
!
       if (maptype .eq. 'G') then
         Xh(0,0) = southwest
         Xh(0,N) = southeast
         Xh(N,0) = northwest
         Xh(N,N) = northeast
       endif
!
       DD = N
       d = N / 2
!
       do is = 1, maxlevel
!  going from grid type I to type II
         delta = delta * 0.5**(0.5 * H)
         do ix = d, N-d, DD
            do iy = d, N-d, DD
               Xh(ix,iy) = f4(delta, Xh(ix+d,iy+d), Xh(ix+d,iy-d),   &
                  Xh(ix-d,iy+d), Xh(ix-d,iy-d))
            enddo
         enddo
!  displace other points also if needed
         if (addition) then
            do ix = 0, N, DD
              do iy = 0, N, DD
                 Xh(ix,iy) = Xh(ix,iy) + delta * Gauss(0.0,sigma,iseed)
              enddo
            enddo
         endif
!  going from grid type II to type I
         delta = delta * 0.5**(0.5*H)
!  interpolate and offset boundary grid points
         do ix = d, N-d, DD
            Xh(ix,0) = f3(delta, Xh(ix+d,0), Xh(ix-d,0), Xh(ix,d))
            Xh(ix,N) = f3(delta, Xh(ix+d,N), Xh(ix-d,N), Xh(ix,N-d))
            Xh(0,ix) = f3(delta, Xh(0,ix+d), Xh(0,ix-d), Xh(d,ix))
            Xh(N,ix) = f3(delta, Xh(N,ix+d), Xh(N,ix-d), Xh(N-d,ix))
            if (wrap) then
               Xh(ix,N) = Xh(ix,0)
               Xh(N,ix) = Xh(0,ix)
            endif
         enddo
!  interpolate and offset inter grid points
         do ix = d, N-d, DD
            do iy = DD, N-d, DD
               Xh(ix,iy) = f4 (delta, Xh(ix,iy+d), Xh(ix,iy-d),   &
                      Xh(ix+d,iy), Xh(ix-d,iy))
            enddo
         enddo
         do ix = DD, N-d, DD
            do iy = d, N-d, DD
               Xh(ix,iy) = f4(delta, Xh(ix,iy+d), Xh(ix,iy-d),    &
                  Xh(ix+d,iy), Xh(ix-d,iy))
            enddo
         enddo
!  displace other points also if needed
         if (addition) then
            do ix = 0, N, DD
               do iy = 0, N, DD
                  Xh(ix,iy) = Xh(ix,iy) + delta * Gauss(0.0,sigma,iseed)
               enddo
            enddo
            do ix = d, N-d, DD
               do iy = d, N-d, DD
                  Xh(ix,iy) = Xh(ix,iy) + delta * Gauss(0.0,sigma,iseed)
               enddo
            enddo
          endif
          DD = DD / 2
          d = d / 2
       enddo
!
       return
       end
!
       function Gauss (xm, xs, iseed)

       use repsblock
       real xm, xs, sum
!
! create a normal number by summing 12 uniform random numbers
       sum = 0.0
       do i = 1,12
           sum = sum + ran1(iseed)
       enddo
       Gauss = (sum - 6.0) * xs + xm
!
       return
       end
!
       subroutine sort (n, ra)
!  sort routine from Numerical Recipes
       real ra(n)
       l = n/2 + 1
       ir = n
!
10     continue
          if (l .gt. 1) then
             l = l-1
             rra = ra(l)
          else
             rra = ra(ir)
             ra(ir) = ra(1)
             ir = ir-1
             if (ir .eq. 1) then
                ra(1) = rra
                return
             endif
          endif
          i = l
          j = l+l
!
20     if (j .le. ir) then
           if (j .lt. ir) then
              if (ra(j) .lt. ra(j+1)) j = j+1
           endif
           if (rra .lt. ra(j)) then
               ra(i) = ra(j)
               i = j
               j = j+j
           else
               j = ir+1
           endif
           goto 20
       endif
!
       ra(i) = rra
       goto 10
!
       end
