      module parmlist
      implicit none
      save
          INTEGER, parameter :: maxprm=1026  ! maximum number of rows and columns = maxprm + 2
          INTEGER, parameter :: maxsp=100     ! maximum number of species
!         INTEGER, parameter :: maxcurd=25   ! maximum number of curds per level
          INTEGER, parameter :: maxsim=10010 ! maximum simulation length
      end module parmlist
!
       module repsblock
       use parmlist
       implicit none
       save
!  mvtype - controls absorbing, reflecting or wrapped boundaries
!  exans  - species extinction check
          CHARACTER* 1 ::  mvtype, exans
!  nrep is the total number of replications 
!  irep is the current simulation number
!  simlen is the length of the simulation
          INTEGER      :: nrep, irep, simlen
!  iseed is a negative random number seed
!    separate seeds are used for map generation, disturbance and movement
!  iseed_map   - map generation seed
!  iseed_dstb  - disturbance generation seed
!  iseed_mov   - movement generation seed
!         INTEGER iseed, iseed_map, iseed_dstb, iseed_mov

!  ivf -- visualization frequency
          INTEGER iseed_map, iseed_dstb, iseed_mov, ivf, ipf
!  p is a vector of probabilities for each habitat type
!  pr is a scratch space for calculated probabilities
          REAL, ALLOCATABLE ::  p(:)
          real              ::  pr
       end module repsblock
!
       MODULE mapblock
       USE parmlist
       IMPLICIT none
       save
!  maptype - habatit map type (i.e., random, curdled, etc.)
!  mapinit - method of initialization of species map (i.e., optimal,
!     random, edge, etc.)
!  gans - answer to question "produce output map?"
          CHARACTER maptype, mapinit, gans
!  mapfile  - name of input map
!  outfile  - name of output map
          CHARACTER*60 mapfile, outfile
!  maxrow and maxcol are the maximum row and column dimensions
!  irow and jcol are the current map row and column dimensions
!  ihab is the number of habitat types
          integer maxrow, maxcol, irow, jcol, ihab
!  map_hab is the habitat map
!  map_sp is the map of species locations
          INTEGER, allocatable :: map_hab(:,:)
          INTEGER, allocatable :: map_sp(:,:)
          INTEGER, ALLOCATABLE :: iwk(:,:)
          INTEGER, ALLOCATABLE :: indx(:)
       END module mapblock
!
!      module curdblock
!      USE parmlist
!      implicit none
!      save
!  this module contains the variables needed to produce a 
!   curdled map
!  icasno is the number of curdled levels
!  ncas is vector of sizes of each curdled level
!         INTEGER, allocatable ::  isl(:), jsl(:), mat(:,:)
!         INTEGER, ALLOCATABLE ::  icdx(:)
!         integer   icasno, isav(6), jsav(6)
!         integer   ncas(6)
!         real      pcas(6), qcas(6)
!  isl and jsl are workspace vectors for recursive curdling
!  pcas is a vector of probabilities for each level
!      end module curdblock
!
       module fractblock
       implicit none
       SAVE 
!  this module contains the parameters for generating fractal maps
!  maxlevel is the number of levels for each map
!    2**maxlevel = size of map
          integer maxlevel, N
!  H is the "roughness" parameter
!  sigma is the variance (set to 1.0)
!  northeast--northwest are the initial values for the map corners
!    used only for wrapped maps or gradients
         INTEGER,  ALLOCATABLE ::  kknt(:)
         REAL,     ALLOCATABLE ::  Xh(:,:), cfd(:), xcrit(:)

         real H, sigma, northeast, southeast, southwest, northwest
         LOGICAL wrap
       end module
!
       module specblock
       USE parmlist
       implicit none
       save
!  this module contains variables describing species characteristics
!  ispecies is the number of species
!  iopt is a matrix of habitat suitabilities
          integer ispecies
          integer iopt(maxsp,maxsp)
!  shmort is a matrix of species (i) and habitat (i) mortalities
          real perx(maxsp)          ! perennial extinction probability
!         REAL shmort(maxsp,maxsp)
!  dc is a species vector of distribution types for dispersal
!  movtype - will species movement be wrapped, absorbed or replected?
          logical immig(maxsp)
          CHARACTER dc(maxsp), life_form(0:maxsp), movtype
!  xbar and sd are the means and standard deviations for dispersal
          real    xbar(maxsp)
          real    sd(maxsp)
          real    alpha             ! inverse abundance exponent
          real    Distance          ! off-site immigration distance
!  rdst is the maximum distances of seed dispersal for each species
!  fecund is the species specific fecundities
          real    rdst(maxsp)
          real    fecund(maxsp)
!  vicdf is the percent of species present on map
!  ab_cdf is the off-site abundance list
          real    vicdf(maxsp), ab_cdf(0:maxsp)
!  sd_rain is the probability of site receiving seeds via a 
!     seed rain from species locally present on the map
          real    sd_rain(maxsp), d_slop(maxsp)
! 
!  the following variables are calculated within the program
!  ncnt - number of neighbors overwhich seeds are dispursed
!  is - matrix of row locations of species
!  js - matrix of column locations of species
          INTEGER, ALLOCATABLE :: ncnt(:)
          INTEGER, ALLOCATABLE :: is(:,:)
          INTEGER, ALLOCATABLE :: js(:,:)
!  defun is matrix of dispersal probabilities (precalculated)
          REAL,    ALLOCATABLE :: dfun(:,:)
       end module
!
       module statblock
       USE parmlist
       implicit none
       save
          integer icdf(maxsp), icnum
!
          real xsum(maxsp+1), xsscp(maxsp+1)
          real xmin(maxsp+1), xmax(maxsp+1)
          real freq, sumin, sumout, t(6)
          REAL loglacun(2), logboxsize(20)
!
       end module
!
       module dstrb_blk
       USE parmlist
       implicit none
       save
           INTEGER  nstack
           INTEGER, allocatable :: istack(:)
           integer, allocatable :: jstack(:)
!  dstb_map is disturbance layer
           INTEGER, ALLOCATABLE :: dstb_map(:,:)
           integer  idefect(maxsim)
!  duration of simulation (idur can not be greater than fract_years * simlen)
           INTEGER :: idur, idnt, max_sevr
           REAL  ::   px, fract_per_yr, fract_years, dcv, dH
           REAL  ::   pb, rdinc, p_irs=0.05
!  px -- probability for cluster growth (range 0.0 to 1.0)
!  fract_per_yr -- average fraction of landscape disturbed per year
!  fract_years -- fraction of years that disturbance occurs
!  freq_type is the method used to generate year-to-year disturbance frequencies
!  d_type is the method for simulation of spatial distribution of disturbance
!  dcv -- coefficient of fariation for disturbance variance
!  dH -- coefficient (rang 0.0 to 1.0) for disturbance fbm algorithm
!  p_irs -- fraction of simulation for initialization (w/o disturbance)
!  max_sevr -- index of maximum severity
           CHARACTER*1 freq_type, d_type
       end module
