! regrow.f90      LINUX version   11/27/04
!   this version of regrow examines a neighborhood defined    |
!   by 'rdst' and calculates the probability of a seed        |
!   falling on each site according to the UNIFORM, NORMAL,    |
!   EXPONENTIAL or INVERSE SQ distances.  The seed lottery    |
!   is then performed with the probabilities modified by the  |
!   habitat optima and fecundity                              |
!
       subroutine regrow (igen)
       USE repsblock
       USE mapblock
       USE specblock
       USE dstrb_blk
!
       implicit none
       integer :: i, ii, j, jj, iloc, jloc, igen, iseed
       real    :: ttl, rx, cfl, cfh, rseed(maxsp), fixt, ds_ratio
       real    :: ran1, rminrain
!
!  save species distribution in a workfile
       iwk = map_sp
!  stochastic immigration from off-site
       immig = .false.
       do i = 1, ispecies
          if (ran1(iseed_mov) .lt. Distance) then
             immig(i) = .true.
          endif
       enddo
!
!  if this is the first generation, find neighborhood to examine
       if (igen .eq. 1) call neighbor
!
!  randomly determine the relative number of "species propagules" 
!    that reach the site -- then use abundance plus suitability rule 
!    to declare winner
       do j = 2, jcol+1
          do i = 2, irow+1
             if (map_hab(i,j) .ne. 0) then
!  check life-form, examine neighborhood and randomly collect propagules
!    and adjust according to species "optimality function"
               if (map_sp(i,j) .gt. 0) then    !  consider random site mortality
                  select case (life_form(map_sp(i,j)))
                     case ('A')               ! annuals
                        map_sp(i,j) = 0
                     case default    !! all others
                        if (ran1(iseed) .lt. perx(map_sp(i,j))) map_sp(i,j) = 0
                  END select
               endif

!  now if site is empty 

               if (map_sp(i,j) .eq. 0) then    ! if map location is empty
                  do ii = 1, ispecies
                     rseed(ii) = 0.0
                     if (vicdf(ii) .gt. 0) then  ! if species ii is present
                        do jj = 1, ncnt(ii)      ! examine neighborhood
                          iloc = is(ii,jj) + i
                          jloc = js(ii,jj) + j

                          select case (movtype)
                              case ('W') ! wrap edges
                                if (iloc .lt. 2) iloc = irow + iloc
                                if (iloc .gt. irow+1) iloc = iloc - irow
                                if (jloc .lt. 2) jloc = jloc + jcol
                                if (jloc .gt. jcol+1) jloc = jloc - jcol

                              case ('A') ! absorbing rows and columns
                                if (iloc .lt. 2) iloc = 1
                                if (iloc .gt. irow+1) iloc = irow + 2
                                if (jloc .lt. 2) jloc = 1
                                if (jloc .gt. jcol+1) jloc = jcol + 2

                              case ('R') ! reflecting rows and columns
                                if (iloc .lt. 2) iloc = iabs(iloc) + 2
                                if (iloc .gt. irow+1) iloc =  &
                                   (irow+1)-(iloc-irow-2)
                                if (jloc .lt. 2) jloc = iabs(jloc) + 2
                                if (jloc .gt. jcol+1) jloc =  &
                                (jcol+1)-(jloc-jcol-2)
   
                              case default
                          end select
!
! examine probabilities ('dfun') to determine number of propagules
!    must use iwk here because that's the old map
                          if (iwk(iloc,jloc) .eq. ii) then
                             rseed(ii) = rseed(ii) + (dfun(ii,jj) * fecund(ii))
                          endif
                        enddo        ! do jj  -- ncnt (neighbors) loop
!
                     end if       ! if species > 0.0


!  SEED RAIN * local (map) abundance
                     rseed(ii) = rseed(ii) + (sd_rain(ii) * vicdf(ii))

! Seed immigration from off-island pool (only if disturbance occurs)
  ! ab_cdf is off-island abundance (set in InitSpc)
  ! ab_cdf is a cumulative distribution -- hence subtraction

                    if (immig(ii)) then
  ! reset minimum seedrain for each species 
                       rminrain = 0.001   
                       if (sd_rain(ii) .gt. rminrain) rminrain = sd_rain(ii)
                       
                       rseed(ii) = rseed(ii) + (rminrain * &
                            (ab_cdf(ii)-ab_cdf(ii-1)))
                    end if
!
!  multiply by habitat optimum ('iopt') and relative fecundity ('fecund')
                    rseed(ii) = rseed(ii) * iopt(ii,map_hab(i,j)) 

!  add disturbance effect as a slope parameter
                    if (freq_type .ne. 'N' .and. d_slop(ii) .gt. 0.0) then
                       ds_ratio = float(dstb_map(i,j)) / float(max_sevr)
                       rseed(ii) = rseed(ii) - rseed(ii) * (d_slop(ii) * ds_ratio)
                    end if
                  enddo           ! do ii  -- species loop

!
!  calculate total number of propagules for site i,j
                 ttl = 0.0
                 do ii = 1, ispecies 
        !  linear local extinction if seed source below criterion
                   if (rseed(ii) .lt. 0.0) rseed(ii) = 0.0
                   if (rseed(ii) .lt. 0.25 .and. &
                       ran1(iseed_mov) .gt. rseed(ii)) then
                        rseed(ii) = 0.0
                     else
                       ttl = ttl + rseed(ii)
                   end if
                 enddo           ! do ii -- species
!
!  spin random number and declare winner by calculating the cumulative
!   probabilities "on the fly"
                 if (ttl .gt. 0.0) then
                    rx = ran1(iseed_mov)
                    cfl = 0.0
                    cfh = 0.0
                    do ii = 1, ispecies
                       cfh = cfh + rseed(ii)/ttl
                       if (rx .gt. cfl .and. rx .le. cfh)  &
                          map_sp(i,j) = ii
                       cfl = cfh
                    enddo          ! do ii -- species
                 end if             ! if (ttl .gt. 0.0)

               END IF      ! if deadsite
             end if        ! if map_hab > 0

          enddo          ! do i -- rows
       enddo             ! do j -- columns
!
       return
       end
