!  stats.f90    
!   ***
       Subroutine StatInit
       USE statblock
       implicit none
!
!  initialize statistical registers
       xsum  = 0.0
       xsscp = 0.0
       xmin  = 1.e9
       xmax  = 0.0
!
       return
       end
!
!  called at end of iteration
       Subroutine AccumStats (ik, isp_rich)
       USE repsblock
       USE mapblock
       USE specblock
       USE statblock
       USE dstrb_blk

       implicit none
       integer i, j, ii, jj, ik, isp_rich
       real vic, value
       real tfreq(0:maxsp,0:maxsp), ttlh(0:maxsp)  ! species-habitat use counters
!
!  initialize the frequency counts for species x habitat matrix
       do i = 0, ispecies
          do j = 0, ihab
             tfreq(i,j) = 0.0
             ttlh(j) = 0.0
          enddo
       enddo
!
!  count frequencies
       do i = 2, irow+1
          do j = 2, jcol+1
             ii = map_sp(i,j)
             jj = map_hab(i,j)
             tfreq(ii,jj) = tfreq(ii,jj) + 1.0
             ttlh(jj) = ttlh(jj) + 1.0
          enddo
       enddo
!
!  normalize and printout frequencies
!       print 101, (i,i=0,ihab)
!       write (15,101) (i,i=0,ihab)
101    format (/t4,'Species   by   Habitat Association ', &
                    '[normalized by habitat]'/ t10,10(i5,5x))
!
       do i = 0, ispecies
          do j = 0, ihab
             if (ttlh(j) .gt. 0) then
                 tfreq(i,j) = tfreq(i,j) / ttlh(j)
               else
                 tfreq(i,j) = 0.0
             endif
          enddo
!
!          print 100, i,(tfreq(i,j),j=0,ihab)
!          write (15,100) i,(tfreq(i,j),j=0,ihab)
100       format (t6,i2,10(2x,f8.5))
       enddo
!
       Call CDF
       do i = 1, ispecies
!
!  normalize to obtain percent
          vic = (float(icdf(i)) / float(irow*jcol))*100.0
!  output to saslog.dat
!           write (10,843)  irep, simlen, fract_per_yr, i, vic, &
!                           fecund(i), sd_rain(i)
843         format (2(i4,1x),g12.6,i3,1x,3(g12.6,1x))

          Call SSCP (i, vic)
!          Call SSCP (ispecies+i,radius(i))
!          rx = float(iedg_time(i))
!          Call SSCP (2*ispecies+i,rx)
       enddo
       value = float(isp_rich)
       call SSCP (ispecies+1, value)

       return
       end
!
       Subroutine contagion (ik, cntg, px, diversity)
       USE mapblock
       USE specblock
!  local variables
       implicit none
       REAL px(0:maxsp), q(0:maxsp,0:maxsp), diversity(4)
       REAL cmax, cntg, xsum, dimx, xispecies
       INTEGER i, j, ii, jj, ispcnt, ik

!  set pi's = 0.0, including px(0)
       px = 0.0
!  set q(i,j)'s = 0.0, including q(0,0)
       q = 0.0

!  estimate counts
       do i = 2, irow+1
          do j = 2, jcol
             ii = map_sp(i,j)
             jj = map_sp(i,j+1)
             px(ii) = px(ii) + 1.0
             q(ii,jj) = q(ii,jj) + 1.0
          enddo
          ii = map_sp(i,jcol+1)
          px(ii) = px(ii) + 1.0    ! count the last row (no contagion)
       enddo

!  estimate contagion
       xispecies = float(ispecies)
       cmax = ispecies * alog(xispecies)
       xsum = 0.0
       do i = 1,ispecies
          do j = 1,ispecies

!  turn counts into frequencies
             if (px(i) .gt. 0.0) then
                  q(i,j) = q(i,j) / px(i)
               else
                  q(i,j) = 0.0
             endif
             if (q(i,j) .gt. 0.0) xsum = q(i,j) * alog(q(i,j)) + xsum
          end do
       end do
       cntg = 1.0 + xsum / cmax

       do i = 1, 4
          diversity(i) = 0.0
       end do

!  normalize P
       dimx = float(irow * jcol)
       ispcnt = 0
       do i = 0, ispecies
          px(i) = px(i) / dimx
          if (px(i) .gt. 0.0 .and. i .gt. 0) then
             diversity(1) = diversity(1) + 1.0
             diversity(2) = diversity(2) + px(i)*log(px(i))
             diversity(3) = diversity(3) + px(i)*px(i)
             ispcnt = ispcnt + 1
          end if
       end do

!  calculate diversity indices
                      ! species richness
       IF (ispcnt .GT. 1) then
          diversity(2) = - diversity(2) / log(float(ispcnt))  ! Shannon
	 else
	  diversity(2) = 0.0
       endif
       diversity(3) = 1.0 - diversity(3)
       diversity(4) = (float(ispecies) * diversity(3) - 1.0) / (float(ispecies) - 1.0)  ! simpson


       return
       end
!
       Subroutine CDF
       USE specblock
       USE mapblock
       USE statblock
       implicit none
       integer i, j, icnt
!
!  look at map and create cumulative frequency distribution
       icdf  = 0
       do i = 2, irow+1
          do j = 2, jcol+1
             if (map_sp(i,j) .ge. 1) then
                icnt = map_sp(i,j)
                icdf(icnt) = icdf(icnt) + 1
             endif
          enddo
       enddo
!
       return
       end
!
! ***
       Subroutine SSCP (ist, value)
       USE statblock
       implicit none
       integer ist
       real value
!
!  obtain sum
       xsum(ist) = xsum(ist) + value
!
!  obtain sum of squares 
       xsscp(ist) = xsscp(ist) + value*value
!
!  find min and max
       if (value .lt. xmin(ist)) xmin(ist) = value
       if (value .gt. xmax(ist)) xmax(ist) = value
!
       return
       end
!
!  called at end of all iterations
       Subroutine StatsOut
       USE repsblock
       USE statblock
       USE specblock
!
       implicit none
       CHARACTER*4 lm(3)
       integer k, j, i
       real suml, xmean, var, std, cv
!
!  output statistical results
       print 840, nrep, simlen
       write (15,840) nrep, simlen
840       format (/20x,'STATISTICAL SUMMARY (N = ',i3,' Final Year =', &
     &i8,')'/' Variable',10x,'Mean'7x,'St.Dev.',5x,'C. V.',5x, &
     &'Minimum',5x,'Maximum')
!
       lm(1) = '%'
       lm(2) = 'rad'
       lm(3) = 'time'
       suml=0.0
!
       do k = 1, ispecies
!         do j = 1, 3
          do j = 1, 1
            i = k+(j*ispecies)-ispecies
            xmean = xsum(i) / nrep
            var   = xsscp(i) - xsum(i)*xsum(i)/nrep
            if (var .gt. 0.0 .and. nrep .gt. 1) then
                std = sqrt (var / (nrep - 1))
              else
                std = 0.0
            endif
            cv = 0.0
            if (std .gt. 0.0) cv = std / xmean * 100.0
            if (xmin(i) .gt. xmean) xmin(i) = xmean
            suml = suml + xmean
            print 841, k, lm(j), xmean, std, cv, xmin(i), xmax(i)
            write (15,841) k, lm(j), xmean, std, cv, xmin(i), xmax(i)
841         format (t3,'sp(',i2,'): ',a4,5(f10.4,2x))
          enddo
       enddo
!  now print species sum
       print 842, suml
       write (15,842) suml
842    format (t6,'Total Sp = ',f10.4)

!  now print species richness
       i = ispecies+1
       lm(j) = 'Sp_R'
       k = 0
       xmean = xsum(i) / nrep
       var   = xsscp(i) - xsum(i)*xsum(i)/nrep
       if (var .gt. 0.0 .and. nrep .gt. 1) then
          std = sqrt (var / (nrep - 1))
        else
          std = 0.0
       endif
       cv = 0.0
       if (std .gt. 0.0) cv = std / xmean * 100.0
       if (xmin(i) .gt. xmean) xmin(i) = xmean
         print 841, k, lm(j), xmean, std, cv, xmin(i), xmax(i)
         write (15,841) k, lm(j), xmean, std, cv, xmin(i), xmax(i)

       return
       end
!
!  called every time step
       Subroutine WriteData (ik, done)
       USE repsblock
       USE mapblock
       USE specblock
       USE statblock
!
       implicit none
       integer ik, i
       REAL px(0:maxsp), diversity(4), cntg
       logical done
!
       if (ik .eq. simlen) done = .true.
!
!  estimate %occupancy and contagion
!      call contagion (ik, cntg, px, diversity)
!      write (11,100) irep, ik, (px(i), i=0,ispecies)
!      write (16,101) irep, ik, cntg, (diversity(i), i=1,4)
100    format (i4,i5,1x,(40(f8.6,1x)))
101    format (' iter=',i3,' yr=',i5,1x,' cntg= ',5(f10.6,1x))

       return
       end
!
!**
       Subroutine WriteLabel
       USE repsblock
       USE specblock
       implicit none
!
!  Write label on output data set
!      write (11,100) SimLen, Ddim, Dfreq, ispecies, nrep
100    format (' SimLen= ',i4,' D_dim= ',i3,' D_freq= ',g12.6, &
               ' Species= ',i3,' Nrep= ',i3)
!      write (11,101) nrep, ispecies, simlen
101    format (4I10,1x,g12.6)
!
       return
       end
